/*
 * k_compat.h 1.8 1995/12/03 01:26:43 (David Hinds)
 *                1998/04/06 modified by Pavel Pisa pisa@CMP.felk.cvut.cz
 */

#ifndef _LINUX_K_COMPAT_H
#define _LINUX_K_COMPAT_H

#define VERSION(v,p,s) (((v)<<16)+(p<<8)+s)

#ifndef LINUX_VERSION_CODE
  #error LINUX_VERSION_CODE not defined
#endif

#if (LINUX_VERSION_CODE < VERSION(1,3,38))

#ifdef MODULE
#include <linux/module.h>
#if !defined(CONFIG_MODVERSIONS) && !defined(__NO_VERSION__)
char kernel_version[] = UTS_RELEASE;
#endif
#else
#define MOD_DEC_USE_COUNT
#define MOD_INC_USE_COUNT
#endif

#else /* 1.3.38 */

#ifdef CONFIG_MODVERSIONS
#define MODVERSIONS 1
#if (LINUX_VERSION_CODE >= VERSION(1,3,40))
#include <linux/modversions.h>
#endif
#endif
#include <linux/module.h>

#endif /* 1.3.38 */

/* I do not know exactly date of change */
#if (LINUX_VERSION_CODE < VERSION(2,1,5))
  #define RWRET      int
  #define RWCOUNT_T  int
  #define RWINODE_P  struct inode *inode,
  #define RWPPOS_P
  #define RWINODE    inode
#else /* >= 2.1.5 */
  #include  <asm/uaccess.h>
  #define get_fs_long(ptr) \
    ({ long  x; /* __typeof__(*(ptr)) x; */ \
       __get_user_check(x,(long  *)(ptr),sizeof(long)); x; })
  #define get_fs_word(ptr) \
    ({ short x; \
       __get_user_check(x,(short *)(ptr),sizeof(short)); x; })
  #define get_fs_byte(ptr) \
    ({ unsigned char x; \
       __get_user_check(x,(unsigned char*)(ptr),sizeof(char)); x; })
  #define put_fs_long(x,ptr) \
    ( __put_user_check((x),(long *)(ptr),sizeof(long)) )
  #define put_fs_word(x,ptr) \
    ( __put_user_check((x),(short *)(ptr),sizeof(short)) )
  #define put_fs_byte(x,ptr) \
    ( __put_user_check((x),(unsigned char*)(ptr),sizeof(char)) )

  #if (LINUX_VERSION_CODE < VERSION(2,1,36))
     #define RWRET      long
     #define RWCOUNT_T  unsigned long
     #define CLOSERET   void
     #define RWINODE_P  struct inode *inode,
     #define RWPPOS_P
     #define RWINODE    inode
  #elif (LINUX_VERSION_CODE < VERSION(2,1,76)) /* may need correction */
     #define CLOSERET   int
     #define RWRET      long
     #define RWCOUNT_T  unsigned long
     #define RWINODE_P  struct inode *inode,
     #define RWPPOS_P
     #define RWINODE    inode
  #else /* >= 2.1.76 */ /* may need correction */
     #define CLOSERET   int
     #define RWRET      ssize_t
     #define RWCOUNT_T  size_t
     #define RWINODE_P
     #define RWPPOS_P   ,loff_t *ppos
     #define RWINODE    file->f_dentry->d_inode
  #endif /* 2.1.36 */

#endif /* 2.1.5 */

/* definition of standard parameters for read/write file functions */
#define READ_PARAMETERS \
	RWINODE_P struct file *file, char *buf, RWCOUNT_T count RWPPOS_P

#define WRITE_PARAMETERS \
	RWINODE_P struct file *file, const char *buf, RWCOUNT_T count RWPPOS_P

#endif /* _LINUX_K_COMPAT_H */
